import jwt

from ievv_auth.ievv_api_key.models import ScopedAPIKey
from ievv_auth.ievv_jwt.backends.base_backend import AbstractBackend
from ievv_auth.ievv_jwt.exceptions import JWTBackendError


class ApiKeyBackend(AbstractBackend):

    @classmethod
    def get_backend_name(cls):
        return 'api-key'

    def make_authenticate_success_response(self, base_payload, *args, **kwargs):
        return {
            'access': self.encode(base_payload=base_payload)
        }



# Generated by Django 3.1.3 on 2020-11-19 19:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('ievv_api_key', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scopedapikey',
            name='base_jwt_payload',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='scopedapikeyauthenticationlog',
            name='log_data',
            field=models.JSONField(default=dict, null=True),
        ),
    ]



from ievv_auth.project.default.settings import *

from django_dbdev.backends.postgres import DBSETTINGS


DATABASES = {
    'default': DBSETTINGS
}
DATABASES['default']['PORT'] = 27253

INSTALLED_APPS += [
    'django_dbdev'
]



from ipware import get_client_ip
from rest_framework import serializers
from rest_framework.exceptions import AuthenticationFailed

from ievv_auth.ievv_api_key.models import ScopedAPIKey
from ievv_auth.ievv_jwt.backends.backend_registry import JWTBackendRegistry
from ievv_auth.ievv_jwt.exceptions import JWTBackendError


class ApiKeyObtainJWTSerializer(serializers.Serializer):
    api_key = serializers.CharField()

    def validate(self, attrs):
        request = self.context.get("request")
        client_ip, is_routable = get_client_ip(request)
        is_valid, instance = ScopedAPIKey.objects.is_valid_with_logging(
            api_key=attrs['api_key'],
            extra={
                'ip': client_ip
            }
        )
        if not is_valid or not instance:
            raise AuthenticationFailed('Api key has expired or is invalid')
        jwt_backend_class = JWTBackendRegistry.get_instance().get_backend(instance.jwt_backend_name)
        if not jwt_backend_class:
            raise AuthenticationFailed('Unknown jwt backend could not authenticate')
        backend = jwt_backend_class()
        return backend.make_authenticate_success_response(
            base_payload={
                **instance.base_jwt_payload,
                'api_key_id': instance.id
            }
        )



import base64
import json
from calendar import timegm
from unittest import mock
from unittest.mock import PropertyMock

import jwt as py_jwt
from django.test import TestCase
from django.utils import timezone

from ievv_auth.ievv_jwt.backends.base_backend import BaseBackend
from ievv_auth.ievv_jwt.exceptions import JWTBackendError


class TestBaseBackend(TestCase):

    def test_base_backend(self):
        backend = BaseBackend()
        jwt = backend.encode()
        print(py_jwt.decode(jwt, verify=False))
        decoded = backend.decode(jwt)
        self.assertIn('exp', decoded)
        self.assertIn('iat', decoded)
        self.assertIn('jti', decoded)

    def test_verify_intercepted_payload_extend_expiration(self):
        backend = BaseBackend()
        jwt = backend.encode()
        [header, _, secret] = jwt.split('.')
        decoded = backend.decode(jwt)
        decoded['exp'] = timegm((timezone.now() + timezone.timedelta(weeks=200)).utctimetuple())
        payload = base64.urlsafe_b64encode(
            json.dumps(
                decoded,
                separators=(',', ':')
            ).encode('utf-8')
        ).decode('utf-8')
        new_jwt = f'{header}.{payload}.{secret}'
        with self.assertRaisesMessage(JWTBackendError, 'Token is invalid or expired'):
            backend.decode(new_jwt, verify=True)

    def test_verify_intercepted_payload_added_additional_scope(self):
        backend = BaseBackend()
        jwt = backend.encode()
        [header, _, secret] = jwt.split('.')
        decoded = backend.decode(jwt)
        decoded['scope'] = 'admin'
        payload = base64.urlsafe_b64encode(
            json.dumps(
                decoded,
                separators=(',', ':')
            ).encode('utf-8')
        ).decode('utf-8')
        new_jwt = f'{header}.{payload}.{secret}'
        with self.assertRaisesMessage(JWTBackendError, 'Token is invalid or expired'):
            backend.decode(new_jwt, verify=True)

    def test_sign_jwt_with_another_secret(self):
        backend = BaseBackend()
        jwt = backend.encode()
        decoded = backend.decode(jwt)
        new_jwt = py_jwt.encode(payload=decoded, key='asdxxxxxxxxxxxxxxxxxxxxxxxxxxx')
        with self.assertRaisesMessage(JWTBackendError, 'Token is invalid or expired'):
            backend.decode(new_jwt, verify=True)

    def test_token_has_expired(self):
        with self.settings(IEVV_JWT={
            'default': {
                'ACCESS_TOKEN_LIFETIME': timezone.timedelta(minutes=0),
            }
        }):
            with mock.patch(
                    'ievv_auth.ievv_jwt.backends.base_backend.BaseBackend.access_token_expiration',
                    new_callable=PropertyMock,
                    return_value=timezone.now() - timezone.timedelta(days=1)):
                backend = BaseBackend()
                jwt = backend.encode()
                with self.assertRaisesMessage(JWTBackendError, 'Token is invalid or expired'):
                    backend.decode(jwt, verify=True)



from rest_framework import generics, status
from rest_framework.response import Response

from ievv_auth.ievv_jwt.api.authentication import AUTH_HEADER_TYPES


class JWTBaseView(generics.GenericAPIView):
    permission_classes = ()
    authentication_classes = ()

    serializer_class = None

    www_authenticate_realm = 'api'

    def get_authenticate_header(self, request):
        return '{0} realm="{1}"'.format(
            AUTH_HEADER_TYPES[0],
            self.www_authenticate_realm,
        )

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        return Response(serializer.validated_data, status=status.HTTP_200_OK)



import os
import json
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'ievv_auth', 'version.json')) as f:
    version = json.loads(f.read())


setup(
    name='ievv-auth',
    description='Authentication modules for the Django framework.',
    version=version,
    author='Appresso developers',
    author_email='post@appresso.no',
    packages=find_packages(exclude=['manage']),
    license='BSD',
    url="https://github.com/appressoas/ievv_auth",
    install_requires=[
        'Django>=3',
        'PyJWT>=1.7.1',
        'djangorestframework',
        'psycopg2',
        'django-ipware'
    ],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved',
        'Operating System :: OS Independent',
        'Programming Language :: Python'
    ]
)
